@@ -1,6 +1,6 @@
 package Thrift::API::HiveClient2;
 {
-  $Thrift::API::HiveClient2::VERSION = '0.009';
+  $Thrift::API::HiveClient2::VERSION = '0.010';
 }
 {
   $Thrift::API::HiveClient2::DIST = 'Thrift-API-HiveClient2';
@@ -184,7 +184,10 @@ sub execute {
             )
         );
         if ( ref $rh eq 'Thrift::API::HiveClient2::TFetchResultsResp' ) {
-            $has_more_rows = $rh->{hasMoreRows};
+
+            # NOTE that currently (july 2013) the hasMoreRows method is broken,
+            # see the explanation in the POD
+            $has_more_rows = $rh->hasMoreRows();
 
             for my $row ( @{ $rh->{results}{rows} || [] } ) {
 
@@ -213,7 +216,7 @@ sub execute {
                     [ map { $row->{colVals}[ $idx++ ]{$_}->value() } @{ $column_keys->{$rv} } ];
             }
         }
-        return $result, $has_more_rows;
+        return wantarray ? ( $result, $has_more_rows ) : ( @$result ? $result : undef );
     }
 }
 
@@ -256,7 +259,7 @@ Thrift::API::HiveClient2 - Perl to HiveServer2 Thrift API wrapper
 
 =head1 VERSION
 
-version 0.009
+version 0.010
 
 =head1 METHODS
 
@@ -290,6 +293,21 @@ rows.
 
     my ($rv, $has_more_rows) = $client->fetch( $rh, <maximum records to retrieve> );
 
+IMPORTANT: The version of HiveServer2 that we use for testing is the one
+bundled with CDH 4.2.1. The hasMoreRows method is currently broken, and always
+returns false. So the right way of obtaining the resultset is to keep using
+fetch() until it returns an empty array. For this reason the behaviour of fetch
+has been altered in scalar context (which becomes the current advised way of
+retrieving the data):
+
+    # $rv will be an arrayref is anything was fetched, and undef otherwise.
+    #
+    while (my $rv = $client->fetch( $rh, <maximum records to retrieve> )) {
+        # ... do something with @$rv
+    }
+
+This is the approach adopted in L<https://github.com/cloudera/hue/blob/master/apps/beeswax/src/beeswax/server/hive_server2_lib.py>
+
 =head1 WARNING
 
 Thrift in Perl currently doesn't support SASL, so authentication needs
